@@ -120,7 +120,7 @@ class NoisyCliffordTSimulator(BackendV1):
         'description': 'An OpenCL based qasm simulator',
         'coupling_map': None,
         'basis_gates': [
-            'id', 'h', 'x', 'y', 'z', 's', 'sdg', 'sx', 'sxdg', 't', 'tdg',
+            'id', 'h', 'x', 'y', 'z', 's', 'sdg', 'sx', 'sxdg', 't', 'tdg', 'rz', 'u1', 'p'
             'cx', 'cy', 'cz', 'swap', 'iswap', 'reset', 'measure', 'barrier'
         ],
         'gates': [{
@@ -189,6 +189,24 @@ class NoisyCliffordTSimulator(BackendV1):
             'conditional': True,
             'description': 'Single-qubit adjoint T gate',
             'qasm_def': 'TODO'
+        }, {
+            'name': 'rz',
+            'parameters': [],
+            'conditional': True,
+            'description': 'Single-qubit Pauli-Z axis rotation gate',
+            'qasm_def': 'TODO'
+        }, {
+            'name': 'u1',
+            'parameters': ['lam'],
+            'conditional': True,
+            'description': 'Single-qubit gate [[1, 0], [0, exp(1j*lam)]]',
+            'qasm_def': 'gate u1(lam) q { U(0,0,lam) q; }'
+        }, {
+            'name': 'p',
+            'parameters': ['theta', 'phi'],
+            'conditional': True,
+            'description': 'Single-qubit gate [[cos(theta), -1j*exp(-1j*phi)], [sin(theta), -1j*exp(1j *phi)*sin(theta), cos(theta)]]',
+            'qasm_def': 'gate r(theta, phi) q { U(theta, phi - pi/2, -phi + pi/2) q;}'
         }, {
             'name': 'cx',
             'parameters': [],
@@ -497,23 +515,20 @@ class NoisyCliffordTSimulator(BackendV1):
         if (lam <= 0.):
             return
 
-        # Random azimuth around Z axis of measurement,
-        # which effectively randomizes depolarization orientation in X-Y plane
+        # Original qubit, Z->X basis
+        self._sim.h(qubit)
+        # Random azimuth around Z axis of measurement
         angleZ = random.uniform(0., 4. * math.pi)
-        # Total depolarization magnitude angle,
-        # where use of hyperbolic trig. is based on empirical exploratory work
-        lamAngle = math.asinh(2 * lam)
-
-        # Azimuth
         self._sim.r(Pauli.PauliZ, angleZ, qubit)
-        # Depolarization magnitude
+        # Total depolarization magnitude angle
+        lamAngle = 2 * math.asin(lam)
         self._sim.r(Pauli.PauliY, lamAngle, qubit)
 
         # Allocate an ancilla
         ancilla = self._sim.num_qubits()
         self._sim.allocate_qubit(ancilla)
         # Partially entangle with the ancilla
-        self._sim.mcx([qubit], ancilla)
+        self._sim.mcr(Pauli.PauliY, lamAngle, [qubit], ancilla)
         # Partially collapse the original state
         self._sim.m(ancilla)
         # The ancilla is fully separable, after measurement.
@@ -522,10 +537,27 @@ class NoisyCliffordTSimulator(BackendV1):
         # Uncompute
         self._sim.r(Pauli.PauliY, -lamAngle, qubit)
         self._sim.r(Pauli.PauliZ, -angleZ, qubit)
+        self._sim.h(qubit)
 
     def _apply_op(self, operation, noise):
         name = operation.name
 
+        # Divert variational phase gates to Clifford, when possible.
+        if name == 'rz' or name == 'u1' or name == 'p':
+            angle = operation.params[0]
+            while angle < 0.:
+                angle = angle + 2. * math.pi
+            while angle >= 2. * math.pi:
+                angle = angle - 2. * math.pi
+            if math.isclose(angle, 0.):
+                return
+            if math.isclose(angle, math.pi):
+                name = 'z'
+            elif math.isclose(angle, math.pi / 2.):
+                name = 's'
+            elif math.isclose(angle, -math.pi / 2.):
+                name = 'sdg'
+
         if (name == 'id') or (name == 'barrier'):
             # Skip measurement logic
             return
@@ -567,6 +599,12 @@ class NoisyCliffordTSimulator(BackendV1):
         elif name == 'tdg':
             self._sim.adjt(operation.qubits[0])
             self.inject_depolarizing_1qb_noise(operation.qubits[0], noise)
+        elif name == 'rz':
+            self._sim.r(Pauli.PauliZ, operation.params[0], operation.qubits[0])
+            self.inject_depolarizing_1qb_noise(operation.qubits[0], noise)
+        elif (name == 'u1') or (name == 'p'):
+            self._sim.u(operation.qubits[0], 0, 0, operation.params[0])
+            self.inject_depolarizing_1qb_noise(operation.qubits[0], noise)
         elif name == 'cx':
             self._sim.mcx(operation.qubits[0:1], operation.qubits[1])
         elif name == 'cy':
